import os
import logging
import re

import dotenv

from core.exceptions import ConfigError
from utils.ip import get_ip
from utils.constants import IP_REGEX

dotenv.load_dotenv()


class BankNodeConfig:
    def __init__(self):
        self.logger = logging.getLogger(__name__)
        # Added for compatibility with python 3.9 instead of using
        # logging.getLevelNamesMapping()
        allowed_levels = ["DEBUG", "INFO", "WARNING", "ERROR", "CRITICAL"]

        ip = os.getenv("IP", get_ip())
        timeout = os.getenv("RESPONSE_TIMEOUT", "5")
        client_idle_timeout = os.getenv("CLIENT_IDLE_TIMEOUT", "60")
        verbosity = os.getenv("VERBOSITY", "INFO")
        scan_port_range = os.getenv("SCAN_PORT_RANGE")

        # Port validation
        if not scan_port_range or scan_port_range == "":
            self.logger.error("Scan port range not defined")
            raise ConfigError("Scan port range not defined")

        range_split = scan_port_range.split(":")

        if len(range_split) != 2 or not range_split[0].isdigit() or not range_split[1].isdigit():
            self.logger.error("Scan port range is not in valid format")
            raise ConfigError("Scan port range is not in valid format")

        # Timeout validation
        if not timeout.isdigit():
            self.logger.warning("Request timeout is invalid - Not a number. Falling back to default timeout 5 seconds")

        if not client_idle_timeout.isdigit():
            self.logger.warning("Client idle timeout is invalid - Not a number. Falling back to default idle timeout 60 seconds")

        # Verbosity
        if verbosity not in allowed_levels:
            self.logger.warning("Unknown verbosity %s. Falling back to default verbosity INFO", verbosity)

        # IP validation
        if not re.match(IP_REGEX, ip):
            raise ConfigError(f"Invalid IP {ip} in configuration")

        self.used_port: int
        self.ip = ip
        self.timeout = int(timeout)
        self.client_idle_timeout = int(client_idle_timeout)
        self.verbosity = verbosity
        self.scan_port_start = int(range_split[0])
        self.scan_port_end = int(range_split[1])

    def to_dict(self):
        return {
            "used_port": self.used_port,
            "ip": self.ip,
            "timeout": self.timeout,
            "client_idle_timeout": self.client_idle_timeout,
            "verbosity": self.verbosity,
            "scan_port_start": self.scan_port_start,
            "scan_port_end": self.scan_port_end,
        }


__all__ = ["BankNodeConfig"]
